@@ -23,8 +23,11 @@
 #include <utility>
 #include <cairomm/surface.h>
 #include <cairomm/fontface.h>
+#include <cairomm/matrix.h>
 #include <cairomm/pattern.h>
 #include <cairomm/path.h>
+#include <cairomm/scaledfont.h>
+#include <cairomm/types.h>
 #include <valarray>
 #include <vector>
 #include <cairo.h>
@@ -33,12 +36,6 @@
 namespace Cairo
 {
 
-typedef cairo_glyph_t Glyph; //A simple struct.
-typedef cairo_font_extents_t FontExtents; //A simple struct.
-typedef cairo_text_extents_t TextExtents; //A simple struct.
-typedef cairo_matrix_t Matrix; //A simple struct. //TODO: Derive and add operator[] and operator. matrix multiplication?
-typedef cairo_rectangle_t Rectangle;
-
 /** Context is the main class used to draw in cairomm. 
  * In the simplest case, create a Context with its target Surface, set its
  * drawing options (line width, color, etc), create shapes with methods like
@@ -163,9 +160,9 @@ public:
    * attributes can be modified if desired, (eg. to create a repeating pattern
    * with Pattern::set_extend()).
    *
-   * @param surface : 	a Surface to be used to set the source pattern
-   * @param x : 	User-space X coordinate for surface origin
-   * @param y : 	User-space Y coordinate for surface origin 
+   * @param surface  	a Surface to be used to set the source pattern
+   * @param x  	User-space X coordinate for surface origin
+   * @param y  	User-space Y coordinate for surface origin 
    */
   void set_source(const RefPtr<Surface>& surface, double x, double y);
 
@@ -705,10 +702,37 @@ public:
   void set_font_matrix(const Matrix& matrix);
   void get_font_matrix(Matrix& matrix) const;
   void set_font_options(const FontOptions& options);
+
+  //TODO: Documentation.
+  /**
+   * @since 1.8
+   */
+  void get_font_options(FontOptions& options) const;
+
+  //TODO: Documentation.
+  /**
+   * @since 1.8
+   */
+  void set_scaled_font(const RefPtr<const ScaledFont>& scaled_font);
+
+  //TODO: Documentation.
+  /**
+   * @since 1.8
+   */
+  RefPtr<ScaledFont> get_scaled_font();
+
+  //TODO: Documentation:
   void show_text(const std::string& utf8);
   void show_glyphs(const std::vector<Glyph>& glyphs);
+  /** @since 1.8 */
+  void show_text_glyphs(const std::string& utf8,
+                        const std::vector<Glyph>& glyphs,
+                        const std::vector<TextCluster>& clusters,
+                        TextClusterFlags cluster_flags);
+
   RefPtr<FontFace> get_font_face();
   RefPtr<const FontFace> get_font_face() const;
+
   void get_font_extents(FontExtents& extents) const;
   void set_font_face(const RefPtr<const FontFace>& font_face);
   void get_text_extents(const std::string& utf8, TextExtents& extents) const;
@@ -778,8 +802,8 @@ public:
   /**
    * Gets the current dash array and offset.
    *
-   * @param dashes return value for the dash array
-   * @param offset return value for the current dash offset
+   * @param dashes return value for the dash array.
+   * @param offset return value for the current dash offset.
    *
    * Since: 1.4
    **/
@@ -791,6 +815,9 @@ public:
    * @param matrix	return value for the matrix
    */
   void get_matrix(Matrix& matrix);
+  /** @since 1.8
+   */
+  Matrix get_matrix() const;
 
   /** Gets the target surface associated with this Context.
    *
@@ -919,7 +946,7 @@ public:
    * content type is the only distinction between this function and push_group()
    * which you should see for a more detailed description of group rendering.
    *
-   * @param content: indicates the type of group that will be created
+   * @param content indicates the type of group that will be created
    */
   void push_group_with_content(Content content);
 
